# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.account_holder import AccountHolder
from lusid.models.investment_portfolio import InvestmentPortfolio
from lusid.models.link import Link
from lusid.models.model_property import ModelProperty
from lusid.models.relationship import Relationship
from lusid.models.version import Version

class InvestmentAccount(BaseModel):
    """
    Representation of an Investment Account on the LUSID API  # noqa: E501
    """
    scope:  Optional[StrictStr] = Field(None,alias="scope", description="The scope in which the Investment Account lies.") 
    identifiers: Optional[Dict[str, ModelProperty]] = Field(default=None, description="Unique client-defined identifiers of the Investment Account.")
    display_name:  Optional[StrictStr] = Field(None,alias="displayName", description="The display name of the Investment Account") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="The description of the Investment Account") 
    account_type:  Optional[StrictStr] = Field(None,alias="accountType", description="The type of the of the Investment Account.") 
    account_holders: Optional[List[AccountHolder]] = Field(default=None, description="The Account Holders of the Investment Account.", alias="accountHolders")
    investment_portfolios: Optional[List[InvestmentPortfolio]] = Field(default=None, description="The Investment Portfolios of the Investment Account.", alias="investmentPortfolios")
    lusid_investment_account_id:  Optional[StrictStr] = Field(None,alias="lusidInvestmentAccountId", description="The unique LUSID Investment Account Identifier of the Investment Account.") 
    properties: Optional[Dict[str, ModelProperty]] = Field(default=None, description="A set of properties associated to the Investment Account.")
    relationships: Optional[List[Relationship]] = Field(default=None, description="A set of relationships associated to the Investment Account.")
    href:  Optional[StrictStr] = Field(None,alias="href", description="The specific Uniform Resource Identifier (URI) for this resource at the requested effective and asAt datetime.") 
    version: Optional[Version] = None
    links: Optional[List[Link]] = None
    __properties = ["scope", "identifiers", "displayName", "description", "accountType", "accountHolders", "investmentPortfolios", "lusidInvestmentAccountId", "properties", "relationships", "href", "version", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> InvestmentAccount:
        """Create an instance of InvestmentAccount from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in identifiers (dict)
        _field_dict = {}
        if self.identifiers:
            for _key in self.identifiers:
                if self.identifiers[_key]:
                    _field_dict[_key] = self.identifiers[_key].to_dict()
            _dict['identifiers'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in account_holders (list)
        _items = []
        if self.account_holders:
            for _item in self.account_holders:
                if _item:
                    _items.append(_item.to_dict())
            _dict['accountHolders'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in investment_portfolios (list)
        _items = []
        if self.investment_portfolios:
            for _item in self.investment_portfolios:
                if _item:
                    _items.append(_item.to_dict())
            _dict['investmentPortfolios'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in relationships (list)
        _items = []
        if self.relationships:
            for _item in self.relationships:
                if _item:
                    _items.append(_item.to_dict())
            _dict['relationships'] = _items
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if scope (nullable) is None
        # and __fields_set__ contains the field
        if self.scope is None and "scope" in self.__fields_set__:
            _dict['scope'] = None

        # set to None if identifiers (nullable) is None
        # and __fields_set__ contains the field
        if self.identifiers is None and "identifiers" in self.__fields_set__:
            _dict['identifiers'] = None

        # set to None if display_name (nullable) is None
        # and __fields_set__ contains the field
        if self.display_name is None and "display_name" in self.__fields_set__:
            _dict['displayName'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if account_type (nullable) is None
        # and __fields_set__ contains the field
        if self.account_type is None and "account_type" in self.__fields_set__:
            _dict['accountType'] = None

        # set to None if account_holders (nullable) is None
        # and __fields_set__ contains the field
        if self.account_holders is None and "account_holders" in self.__fields_set__:
            _dict['accountHolders'] = None

        # set to None if investment_portfolios (nullable) is None
        # and __fields_set__ contains the field
        if self.investment_portfolios is None and "investment_portfolios" in self.__fields_set__:
            _dict['investmentPortfolios'] = None

        # set to None if lusid_investment_account_id (nullable) is None
        # and __fields_set__ contains the field
        if self.lusid_investment_account_id is None and "lusid_investment_account_id" in self.__fields_set__:
            _dict['lusidInvestmentAccountId'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        # set to None if relationships (nullable) is None
        # and __fields_set__ contains the field
        if self.relationships is None and "relationships" in self.__fields_set__:
            _dict['relationships'] = None

        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> InvestmentAccount:
        """Create an instance of InvestmentAccount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return InvestmentAccount.parse_obj(obj)

        _obj = InvestmentAccount.parse_obj({
            "scope": obj.get("scope"),
            "identifiers": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("identifiers").items()
            )
            if obj.get("identifiers") is not None
            else None,
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "account_type": obj.get("accountType"),
            "account_holders": [AccountHolder.from_dict(_item) for _item in obj.get("accountHolders")] if obj.get("accountHolders") is not None else None,
            "investment_portfolios": [InvestmentPortfolio.from_dict(_item) for _item in obj.get("investmentPortfolios")] if obj.get("investmentPortfolios") is not None else None,
            "lusid_investment_account_id": obj.get("lusidInvestmentAccountId"),
            "properties": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "relationships": [Relationship.from_dict(_item) for _item in obj.get("relationships")] if obj.get("relationships") is not None else None,
            "href": obj.get("href"),
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

InvestmentAccount.update_forward_refs()
