# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class AggregationMeasureFailureDetail(BaseModel):
    """
    AggregationMeasureFailureDetail
    """
    id:  Optional[StrictStr] = Field(None,alias="id") 
    effective_at: Optional[datetime] = Field(default=None, alias="effectiveAt")
    measure:  Optional[StrictStr] = Field(None,alias="measure") 
    reason:  Optional[StrictStr] = Field(None,alias="reason") 
    detail:  Optional[StrictStr] = Field(None,alias="detail") 
    __properties = ["id", "effectiveAt", "measure", "reason", "detail"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AggregationMeasureFailureDetail:
        """Create an instance of AggregationMeasureFailureDetail from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if measure (nullable) is None
        # and __fields_set__ contains the field
        if self.measure is None and "measure" in self.__fields_set__:
            _dict['measure'] = None

        # set to None if reason (nullable) is None
        # and __fields_set__ contains the field
        if self.reason is None and "reason" in self.__fields_set__:
            _dict['reason'] = None

        # set to None if detail (nullable) is None
        # and __fields_set__ contains the field
        if self.detail is None and "detail" in self.__fields_set__:
            _dict['detail'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AggregationMeasureFailureDetail:
        """Create an instance of AggregationMeasureFailureDetail from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AggregationMeasureFailureDetail.parse_obj(obj)

        _obj = AggregationMeasureFailureDetail.parse_obj({
            "id": obj.get("id"),
            "effective_at": obj.get("effectiveAt"),
            "measure": obj.get("measure"),
            "reason": obj.get("reason"),
            "detail": obj.get("detail")
        })
        return _obj

AggregationMeasureFailureDetail.update_forward_refs()
