# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class AggregationType(str, Enum):
    """
    A list of types, that define the expected output types found from an aggregation request in its result.
    """

    """
    allowed enum values
    """
    STRING = 'String'
    INT = 'Int'
    DECIMAL = 'Decimal'
    DATETIME = 'DateTime'
    BOOLEAN = 'Boolean'
    RESULTVALUE = 'ResultValue'
    RESULT0D = 'Result0D'
    JSON = 'Json'

    @classmethod
    def from_json(cls, json_str: str) -> AggregationType:
        """Create an instance of AggregationType from a JSON string"""
        return AggregationType(json.loads(json_str))
