# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class TransactionRoles(str, Enum):
    """
    TransactionRoles
    """

    """
    allowed enum values
    """
    NONE = 'None'
    LONGLONGER = 'LongLonger'
    LONGSHORTER = 'LongShorter'
    SHORTSHORTER = 'ShortShorter'
    SHORTER = 'Shorter'
    SHORTLONGER = 'ShortLonger'
    LONGER = 'Longer'
    ALLROLES = 'AllRoles'

    @classmethod
    def from_json(cls, json_str: str) -> TransactionRoles:
        """Create an instance of TransactionRoles from a JSON string"""
        return TransactionRoles(json.loads(json_str))
