# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class PostCloseActivity(BaseModel):
    """
    PostCloseActivity
    """
    entity_type:  StrictStr = Field(...,alias="entityType", description="The type of the entity, possible values are: * `PortfolioTransaction`, * `Instrument`, * `InstrumentEvent`, * `InstrumentEventInstruction`, * `PortfolioSettlementInstruction`, and, * `Quote`.") 
    entity_unique_id:  StrictStr = Field(...,alias="entityUniqueId", description="The entity unique ID. The expected format for each entity is: | entityType                       | entityUniqueId                                    | |----------------------------------|---------------------------------------------------| | `PortfolioTransaction`           | `portfolioUniqueId_transactionId`                 | | `Instrument`                     | `instrumentUniqueId`                              | | `InstrumentEvent`                | `corporateActionSourceUniqueId_instrumentEventId` | | `InstrumentEventInstruction`     | `portfolioUniqueId_instructionId`                 | | `PortfolioSettlementInstruction` | `portfolioUniqueId_settlementInstructionId`       | | `Quote`                          | `quoteSeriesUniqueId_quoteSeriesInstrumentId`     |") 
    as_at: datetime = Field(description="The `AsAt` time of the event that needs to be added to the closed period.", alias="asAt")
    effective_at:  Optional[StrictStr] = Field(None,alias="effectiveAt", description="The `EffectiveAt` time of the event that need to be added to the closed period. This can be a date or cut label. Only applicable for `Quote` post-close activities.") 
    __properties = ["entityType", "entityUniqueId", "asAt", "effectiveAt"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PostCloseActivity:
        """Create an instance of PostCloseActivity from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if effective_at (nullable) is None
        # and __fields_set__ contains the field
        if self.effective_at is None and "effective_at" in self.__fields_set__:
            _dict['effectiveAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PostCloseActivity:
        """Create an instance of PostCloseActivity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PostCloseActivity.parse_obj(obj)

        _obj = PostCloseActivity.parse_obj({
            "entity_type": obj.get("entityType"),
            "entity_unique_id": obj.get("entityUniqueId"),
            "as_at": obj.get("asAt"),
            "effective_at": obj.get("effectiveAt")
        })
        return _obj

PostCloseActivity.update_forward_refs()
